package com.incubator.game.contest;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.game.player.Player;
import org.slf4j.Logger;

import java.util.Map;
import java.util.concurrent.*;

/**
 * 官方比赛 管理类
 */
public class ContestService {

    protected Logger logger = Log4jUtil.getLogger(getClass());

    private static ContestService instance;
    /** 比赛对象池 **/
    public ContestPool contestPool;
    /** 比赛缓存 **/
    public Map<Integer, Contest> contestMap = new ConcurrentHashMap<>();

    public synchronized static ContestService getInstance() {
        if (instance == null) {
            instance = new ContestService();
        }
        return instance;
    }

    /**
     * 初始化
     *
     * @param maxIdleContest 最大空闲数量
     */
    public void init(int maxIdleContest) {
        logger.info("初始化官方比赛服务...");
        // 初始化对象池
        this.contestPool = new ContestPool(maxIdleContest);
        //todo 测试创建一个4人比赛，2分钟后开始
        long startTime = System.currentTimeMillis() + (2 * 60 * 1000);
        this.creatContest(1, 1001, startTime);
    }

    /**
     * 获取玩家报名的比赛
     *
     * @param playerId
     * @return
     */
    public Contest getContestByPlayerId(String playerId) {
        for (Contest contest : this.contestMap.values()) {
            if (contest != null && contest.actors != null && contest.actors.containsKey(playerId)) {
                return contest;
            }
        }
        return null;
    }

    /**
     * 创建比赛
     *
     * @param roomType  比赛房间玩法类型
     * @param contestId 比赛id
     * @param startTime 比赛开始时间
     * @return
     */
    public synchronized Contest creatContest(int roomType, int contestId, long startTime) {
        // 从对象池获取比赛实例
        Contest contest = this.contestPool.acquireContest(roomType);
        // 初始化比赛数据
        contest.init(roomType, contestId, startTime);
        // 开始主线程逻辑
        contest.startContestTask();
        // 加入缓存
        this.contestMap.put(contest.data.contestId, contest);
        return contest;
    }

    /**
     * 报名加入比赛
     *
     * @param player
     * @param contestId
     */
    public synchronized Contest joinContest(Player player, int contestId) {
        Contest contest = this.contestMap.getOrDefault(contestId, null);
        if (contest == null) {
            return null;
        }
        // 执行玩家加入比赛逻辑
        contest.joinContest(player);
        // 绑定官方比赛
        player.data.contestId = contestId;
        return contest;
    }
}
