package com.incubator.game.handler.club;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.ZjClubMemberPo;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.message.proto.CommonProto;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-09 14:27
 * 同意或拒绝加入俱乐部
 */
@MessageHandler(id = CommonProto.Cmd.AgreeOrRefuseJoinReq_VALUE)
public class AgreeOrRefuseJoin extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.AgreeOrRefuseJoinRes_VALUE);

        String ids = request.getDataValue("userIds", String.class, "");
        String status = request.getDataValue("status", String.class, "");
        String clubId = request.getDataValue("clubId", String.class, "");

        // 查询指定俱乐部
        String clubListStr = RedisUtil.get(RedisKeyConstant.CLUB_BASE);
        JSONObject clubJson = JSONUtil.parseObj(clubListStr);
        JSONArray clubListArray = clubJson.getJSONArray("clubList");
        List<ZjClubPo> clubList = clubListArray.toList(ZjClubPo.class);
        ZjClubPo zjClub = clubList.stream()
                .filter(club -> clubId.equals(club.getClubId()))
                .findFirst()
                .orElse(null);
        if (zjClub == null) {
            response.setCode(102);
            response.setMessage("俱乐部不存在");
            return;
        }

        // 提取status属性为"0"的元素，并组成新的集合
        String memberListStr = RedisUtil.get(RedisKeyConstant.CLUB_MEMBER + clubId);
        JSONObject memberObject = JSONUtil.parseObj(memberListStr);
        JSONArray clubMemberArray = memberObject.getJSONArray("clubMember");
        List<ZjClubMemberPo> clubMember = clubMemberArray.toList(ZjClubMemberPo.class);
        Map<String, ZjClubMemberPo> memberMap = new HashMap<>();
        for (ZjClubMemberPo member : clubMember) {
            if ("0".equals(member.getMemberStatus())) {
                memberMap.put(member.getUserId(), member);
            }
        }

        String[] userIds = ids.split(",");
        Date now = new Date(); // 创建一次，循环中重用
        for (String userId : userIds) {
            ZjClubMemberPo member = memberMap.get(userId);
            if (member != null) {
                member.setMemberStatus(status);
                if ("1".equals(status)) {
                    zjClub.setClubMemberNum(zjClub.getClubMemberNum() + 1);
                    member.setIsMaster("N");
                    member.setJoinTime(now);

                    String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
                    JSONObject userObject = JSONUtil.parseObj(userInfo);
                    ZjUserLobbyPo lobbyUser = JSONUtil.toBean(userObject.getJSONObject("lobbyUser"), ZjUserLobbyPo.class);
                    if (lobbyUser.getUserClub() == null || lobbyUser.getUserClub().isEmpty()) {
                        lobbyUser.setUserClub(clubId);
                        userObject.put("lobbyUser", lobbyUser);
                        RedisUtil.set(RedisKeyConstant.USER_BASE + userId, userObject.toString());
                    }
                }
            }
        }

        memberObject.put("clubMember", new JSONArray(memberMap.values()));
        RedisUtil.set(RedisKeyConstant.CLUB_MEMBER + clubId, memberObject.toString());
        RedisUtil.set(RedisKeyConstant.CLUB_BASE, clubJson.toString());
        response.setMessage("操作成功！");
    }
}
