package com.incubator.game.handler.club;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.ZjClubMemberPo;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.game.util.UtilCore;
import com.incubator.message.proto.CommonProto;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-09 14:27
 * 创建俱乐部
 */
@MessageHandler(id = CommonProto.Cmd.CreateClubReq_VALUE)
public class CreateClub extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.CreateClubRes_VALUE);

        String userId = request.getDataValue("userId", String.class, "");
        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        JSONObject userObject = JSONUtil.parseObj(userInfo);
        String clubInfo = request.getDataValue("clubInfo", String.class, "");
        ZjClubPo club = JSONUtil.parseObj(clubInfo).toBean(ZjClubPo.class);
        JSONObject jsonObject = getClubListFromRedis();
        JSONArray clubArray = jsonObject.getJSONArray("clubList");
        List<ZjClubPo> clubList = clubArray.toList(ZjClubPo.class);

        if (club.getClubId()==null){

            if (isUserAlreadyCreatedClub(clubList, userId)) {
                response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
                response.setMessage("您已创建过俱乐部");
                return;
            }

            ZjClubPo newClub = createNewClub(club, userId);
            addClubToRedis(jsonObject, clubArray, newClub);//添加俱乐部到redis

            addClubMemberToRedis(userObject, newClub);//添加俱乐部成员到redis

            updateUserLobbyClub(userId, newClub);//设置主俱乐部
        }else {
            Optional<ZjClubPo> existingClub = clubList.stream()
                    .filter(clubPo -> clubPo.getClubId().equals(club.getClubId()))
                    .findFirst();
            existingClub.ifPresent(c -> {
                c.setClubName(club.getClubName());
                c.setClubImg(club.getClubImg());
                c.setClubDescription(club.getClubDescription());
                c.setClubAddress(club.getClubAddress());
            });
            jsonObject.put("clubList", clubList);
            RedisUtil.set(RedisKeyConstant.CLUB_BASE, jsonObject.toString());
        }
    }

    private void addClubMemberToRedis(JSONObject userObject, ZjClubPo newClub) {
        Object lobbyUserObj = userObject.get("lobbyUser");
        ZjUserLobbyPo lobbyUser = JSONUtil.toBean((JSONObject) lobbyUserObj, ZjUserLobbyPo.class);
        ZjClubMemberPo zjClubMemberPo = new ZjClubMemberPo();
        zjClubMemberPo.setMemberId(UtilCore.getDateRandomId());
        zjClubMemberPo.setUserId(lobbyUser.getUserId());
        zjClubMemberPo.setClubId(newClub.getClubId());
        zjClubMemberPo.setIsMain("Y");
        zjClubMemberPo.setIsMaster("Y");
        zjClubMemberPo.setMemberStatus("1");
        zjClubMemberPo.setAvatarUrl(lobbyUser.getAvatarUrl());
        zjClubMemberPo.setNickName(lobbyUser.getNickName());
        zjClubMemberPo.setJoinTime(new Date());
        zjClubMemberPo.setSevenDayMasterScore((Integer) userObject.get("sevenScore"));
        zjClubMemberPo.setRequestExplain("掌门人");
        zjClubMemberPo.setRequestChannel("2");
        RedisUtil.set(RedisKeyConstant.CLUB_MEMBER + newClub.getClubId(), zjClubMemberPo);
    }

    private JSONObject getClubListFromRedis() {
        String clubListStr = RedisUtil.get(RedisKeyConstant.CLUB_BASE);
        return JSONUtil.parseObj(clubListStr);
    }

    private boolean isUserAlreadyCreatedClub(List<ZjClubPo> clubList, String userId) {
        return clubList.stream().anyMatch(club -> club.getClubCreator().equals(userId));
    }

    private ZjClubPo createNewClub(ZjClubPo newClub , String userId) {
        newClub.setClubId(UtilCore.getDateRandomId());
        newClub.setClubCreator(userId);
        newClub.setOnlineCount(1);
        newClub.setClubStatus("1");
        newClub.setClubLevel(1);
        return newClub;
    }

    private void addClubToRedis(JSONObject jsonObject, JSONArray clubArray, ZjClubPo newClub) {
        clubArray.add(newClub);
        jsonObject.set("clubList", clubArray);
        RedisUtil.set(RedisKeyConstant.CLUB_BASE, jsonObject.toString());
    }

    private void updateUserLobbyClub(String userId, ZjClubPo newClub) {
        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        JSONObject userObject = JSONUtil.parseObj(userInfo);
        Object lobbyUserObj = userObject.get("lobbyUser");
        ZjUserLobbyPo lobbyUser = JSONUtil.toBean((JSONObject) lobbyUserObj, ZjUserLobbyPo.class);
        lobbyUser.setUserClub(newClub.getClubId());
        userObject.put("lobbyUser", lobbyUser);
        RedisUtil.set(RedisKeyConstant.USER_BASE + userId, userObject.toString());
    }
}
